/*
 * Copyright (C) 2004, 2005 Filip Pizlo. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY FILIP PIZLO ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL FILIP PIZLO OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include "tsf_test.h"
#include "tsf.h"
#include "write_cases.h"

#include <stdio.h>
#include <ctype.h>

static tsf_stream_file_output_t *output;

static void doit(tsf_reflect_t *r) {
    tsf_buffer_t *buf;
    
    TAT(buf=tsf_reflect_make_buffer(r));
    
    tsf_reflect_destroy(r);
    
    TAT(tsf_stream_file_output_write(output,buf));
    
    tsf_buffer_destroy(buf);
}

int main(int c,char **v) {
    if (c<3) {
        fprintf(stderr,"Usage: %s <output file name> <use comments>\n",v[0]);
        return 1;
    }
    
    output=tsf_stream_file_output_open(v[1],tsf_false,NULL);
    if (output==NULL) {
        fprintf(stderr,"Could not open file for writing: %s\n",
                       tsf_get_error());
        return 1;
    }
    
    TEST_INIT_NOFORK(c,v);
    
    do_tests(tolower(v[2][0])=='t',doit);
    
    if (!tsf_stream_file_output_close(output)) {
        fprintf(stderr,"Could not close file: %s\n",
                       tsf_get_error());
        return 1;
    }
    
    TEST_DONE_CUSTOM(" Done.");
    
    return 0;
}



