declare Operand: integer // negative means a global reference, non-negative means a local reference
declare Type: string
declare CodeOffset: integer

declare VariableDecl: struct {
    type: Type
    name: string
}

declare Instruction: choice {
    nop: void
    
    mov: struct {
        dest: Operand
        src: Operand
    }
    
    add: struct {
        dest: Operand
        src: Operand
    }
    
    alloc: struct {
        type: Type
    }
    
    ret: struct {
        src: Operand
    }
    
    jump: struct {
        target: CodeOffset
    }
    
    call: struct {
        dest: Operand
        callee: Operand
        args: array of Operand
    }
    
    branchZero: struct {
        src: Operand
        target: CodeOffset
    }
}

declare DebugData : struct {
    startOffset: integer
    spanSize: integer
    data: string
}

declare Procedure : struct {
    name: string
}

declare ProcedureDecl: struct {
    pull Procedure
}

declare ProcedureDefn: struct {
    pull Procedure
    variables: array of VariableDecl
    code: array of Instruction
    debug: array of DebugData
}

declare ProgramElement: choice {
    procedureDecl: ProcedureDecl
    procedureDefn: ProcedureDefn
}

declare Program: struct {
    globals: array of ProgramElement
}

