/*
 * Copyright (C) 2003, 2004, 2005 Filip Pizlo. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY FILIP PIZLO ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL FILIP PIZLO OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include "tsf_internal_config.h"
#include "tsf_build_defines.h" /* not a real header file; gives us DYNCC_BUILD_CMD_SUFFIX */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static const char *usage_msg=
"Usage: tsf_config <commands> ...\n"
"Where the available commands are:\n"
"  --cppflags     Lists the flags you would want to pass to a C preprocessor\n"
"                 when compiling a program that uses TSF.\n"
"  --cflags       Lists the flags you would want to pass to a C compiler when\n"
"                 compiling a program that uses TSF.\n"
"  --cxxflags     Lists the flags you would want to pass to a C++ compiler when\n"
"                 compiling a program that uses TSF.\n"
"  --ldflags      Lists the flags you would want to pass to the linker when\n"
"                 linking a program that uses TSF.\n"
"  --libs         Lists the library link commands that you would want to pass to\n"
"                 the linker when linking a program that uses TSF.\n"
"  --prefix       Prints the TSF install prefix.\n"
"  --version      Prints the TSF version that this program corresponds to.\n";

void usage() {
    fprintf(stderr,"%s",usage_msg);
    exit(1);
}

int main(int c,char **v) {
    int i;
    int print_usage=0;
    
    for (i=1;i<c;++i) {
        if (!strcmp(v[i],"--cppflags")) {
            printf("-I%s %s %s\n",INCLUDE_DIR,XTRA_CPPFLAGS,CPPFLAGS);
        } else if (!strcmp(v[i],"--cflags")) {
            printf("%s\n",CFLAGS);
        } else if (!strcmp(v[i],"--cxxflags")) {
            printf("%s\n",CFLAGS);
        } else if (!strcmp(v[i],"--ldflags")) {
            printf("-L%s %s %s\n",LIB_DIR,XTRA_LDFLAGS,LDFLAGS);
        } else if (!strcmp(v[i],"--libs")) {
            printf("-ltsf %s %s\n",XTRA_LIBS,LIBS);
        } else if (!strcmp(v[i],"--prefix")) {
            printf("%s\n",PREFIX);
        } else if (!strcmp(v[i],"--version")) {
            printf("%s\n",PACKAGE_STRING);
        } else {
            fprintf(stderr,"%s is not an option.\n",v[i]);
            print_usage=1;
        }
    }
    
    if (print_usage) {
        fprintf(stderr,"\n");
    }
    
    if (c<2 || print_usage) {
        usage();
    }
    
    return 0;
}


