/*
 * Copyright (C) 2011 Filip Pizlo. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY FILIP PIZLO ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL FILIP PIZLO OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include "tsf.h"
#include "tsf_util.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>

static void usage() {
    fprintf(stderr,"Usage: tsf_sha1sum < input\n");
    fprintf(stderr,"   or: tsf_sha1sum <filename> [<filename2> ...]\n");
    fprintf(stderr,"Without arguments, computes the SHA1 sum of standard input.  With\n");
    fprintf(stderr,"arguments, computes the SHA1 sums of a list of files.\n");
    exit(1);
}

static void printsum(tsf_sha1_wtr_t *sha1,
                     const char *comment) {
    const char *result;
    CHECKTSF(result=tsf_sha1_sum_to_str(tsf_sha1_writer_finish(sha1)));
    printf("%s  %s\n",result,comment);
    free((void*)result);
}

static void dosum(int fd,const char *comment) {
    tsf_sha1_wtr_t *sha1;
    CHECKTSF(sha1=tsf_sha1_writer_create());
    
    for (;;) {
        char buf[4096];
        int res;
        
        res=read(fd,buf,sizeof(buf));
        CHECKSYS(res);
        
        if (res==0) {
            break;
        }
        
        CHECKTSF(tsf_sha1_writer_write(sha1,buf,res));
    }
    
    printsum(sha1,comment);
    
    tsf_sha1_writer_destroy(sha1);
}

int main(int c,char **v) {
    if (c==1) {
        dosum(0,"-");
    } else {
        int i;
        
        if (!strcmp(v[1],"--help") ||
            !strcmp(v[1],"-h") ||
            !strcmp(v[1],"-help")) {
            usage();
        }
        
        for (i=1;i<c;++i) {
            int fd=open(v[i],O_RDONLY);
            CHECKSYS(fd);
            
            dosum(fd,v[i]);
            
            close(fd);
        }
    }
    
    return 0;
}

